-- Created by NuclearTonic
-- The use of this code requires the permission of the author.
-- Permission can be atained by contacting NuclearTonic at: TonicBars@gmail.com
--
-- RESPECT!

import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Tonic.Utils.Class";
import "Tonic.UI.CheckBox";
import "Tonic.UI.ComboBox";
import "Tonic.UI.AutoListBox";
import "Tonic.UI.MenuUtils";
import "Tonic.TonicBars.Menu.QuickslotsMenu";
import "Tonic.TonicBars.Menu.ExtensionsMenu";
import "Tonic.TonicBars.Menu.InventoryMenu";
import "Tonic.TonicBars.Menu.GeneralMenu";
import "Tonic.TonicBars.Menu.AboutMenu";
import "Tonic.TonicBars.Menu.EasyBarMenu";

buttonWidth = 90;
selectionWidth = 160;
selectionHeight = 20;
commandsHeight = 50;
startFromTop = 39;
selectedColor = Turbine.UI.Color( 0.2, 0.2, 0.2 );
selectedColor.A = 1.0;
unselectedColor = Turbine.UI.Color( 0.0, 0.0, 0.0 );
unselectedColor.A = 1.0;
groupingColor = Turbine.UI.Color( 0.5, 0.5, 0.5 );
groupingColor.A = 1.0;
windowWidth = 2000;
windowHeight = 2000;
insideBoxXOffset = selectionWidth + 5;
insideBoxYOffset = startFromTop + 200;

MainMenu = class( Turbine.UI.ListBox );

function MainMenu:Constructor()
	Turbine.UI.ListBox.Constructor( self );
	self:SetWidth( windowWidth );
	self:SetHeight( windowHeight );
	self:SetVisible( true );
	self:SetOrientation( Turbine.UI.Orientation.Horizontal );
	
	shellCommand = Turbine.ShellCommand()
	function shellCommand:Execute(command, arguments)
		Turbine.PluginManager.ShowOptions(Plugins["TonicBars"]);
	end
	Turbine.Shell.AddCommand('tb;tonicbars;TonicBars', shellCommand);

	self.utils = Tonic.UI.MenuUtils();

	self:AddSelectionTypes();

	self.easybarMenu = Tonic.TonicBars.Menu.EasyBarMenu();
	self.quickslotsMenu = Tonic.TonicBars.Menu.QuickslotsMenu();
	self.extensionsMenu = Tonic.TonicBars.Menu.ExtensionsMenu();
	self.inventoryMenu = Tonic.TonicBars.Menu.InventoryMenu();
	self.generalMenu = Tonic.TonicBars.Menu.GeneralMenu();
	self.aboutMenu = Tonic.TonicBars.Menu.AboutMenu();

	self:SetupMainMenuDisplay( EASYBAR_MODE );
end

function MainMenu:AddSelectionTypes()
	self.selectionBox = self.utils:AddAutoListBox( self, Turbine.UI.Orientation.Vertical );

	self.utils:AddLabelBox( self.selectionBox, "", 50, 10 );

	local editButton = Turbine.UI.Lotro.Button();
	editButton:SetText( LOCALESTRINGS.MainButtons["Edit Mode"] );
	editButton:SetSize( buttonWidth + 50, selectionHeight + 50 );
	editButton.MouseClick = function( sender, args )
		local settings = theSettingsManager:GetSettings();
		if ( settings.barMode == NORMAL_MODE and self.priorBarMode == nil ) then
			settings.barMode = EASYBAR_MODE;
		elseif ( settings.barMode == NORMAL_MODE and self.priorBarMode ~= nil ) then
			settings.barMode = self.priorBarMode;
		else
			self.priorBarMode = settings.barMode;
			settings.barMode = NORMAL_MODE;
		end
		maitre_d:RefreshBars();

	end
	self.selectionBox:AddItem( editButton );

	self.utils:AddLabelBox( self.selectionBox, "", 50, 50 );

	local easybarsButton = Turbine.UI.Lotro.Button();
	easybarsButton:SetText( LOCALESTRINGS.MainButtons["Easy Bars"] );
	easybarsButton:SetSize( buttonWidth + 30, selectionHeight );
	easybarsButton.MouseClick = function( sender, args )
		self.easybarMenu:ClearSelection();
		self:SetupMainMenuDisplay( EASYBAR_MODE );
	end
	self.selectionBox:AddItem( easybarsButton );

	local quickbarsButton = Turbine.UI.Lotro.Button();
	quickbarsButton:SetText( LOCALESTRINGS.MainButtons["Quick Slots"] );
	quickbarsButton:SetSize( buttonWidth + 30, selectionHeight );
	quickbarsButton.MouseClick = function( sender, args )
		self.quickslotsMenu:ClearSelection();
		self:SetupMainMenuDisplay( QUICKSLOT_MODE );
	end
	self.selectionBox:AddItem( quickbarsButton );
	
	local extensionButton = Turbine.UI.Lotro.Button();
	extensionButton:SetText( LOCALESTRINGS.MainButtons["Extensions Bars"] );
	extensionButton:SetSize( buttonWidth + 30, selectionHeight );
	extensionButton.MouseClick = function( sender, args )
		self.extensionsMenu:ClearSelection();
		self:SetupMainMenuDisplay( EXTENSION_MODE );
	end
	self.selectionBox:AddItem( extensionButton );
		
	local windowButton = Turbine.UI.Lotro.Button();
	windowButton:SetText( LOCALESTRINGS.MainButtons["Inventory Bars"] );
	windowButton:SetSize( buttonWidth + 30, selectionHeight );
	windowButton.MouseClick = function( sender, args )
		self.inventoryMenu:ClearSelection();
		self:SetupMainMenuDisplay( INVENTORY_MODE );
	end
	self.selectionBox:AddItem( windowButton );
	
	local genericButton = Turbine.UI.Lotro.Button();
	genericButton:SetText( LOCALESTRINGS.MainButtons["General Settings"] );
	genericButton:SetSize( buttonWidth + 30, selectionHeight );
	genericButton.MouseClick = function( sender, args )
		self:SetupMainMenuDisplay( GENERAL_MODE );
	end
	self.selectionBox:AddItem( genericButton );
	
	local aboutButton = Turbine.UI.Lotro.Button();
	aboutButton:SetText( LOCALESTRINGS.MainButtons["About TonicBars"] );
	aboutButton:SetSize( buttonWidth + 30, selectionHeight );
	aboutButton.MouseClick = function( sender, args )
		self:SetupMainMenuDisplay( ABOUT_MODE );
	end
	self.selectionBox:AddItem( aboutButton );
	
	self:AddItem(self.selectionBox);
end

function MainMenu:SetupMainMenuDisplay( mode )
	local settings = theSettingsManager:GetSettings();
	if ( mode == EASYBAR_MODE ) then
		self:ClearItems();
		self:AddItem( self.selectionBox );
		self:AddItem( self.easybarMenu );	
		self.easybarMenu:DisplaySettings( true );
		if ( settings.barMode ~= NORMAL_MODE ) then
			settings.barMode = QUICKSLOT_MODE;
		else
			self.priorBarMode = QUICKSLOT_MODE;
		end
	elseif ( mode == QUICKSLOT_MODE ) then
		self:ClearItems();
		self:AddItem( self.selectionBox );
		self:AddItem( self.quickslotsMenu );
		
		if ( self.quickslotsMenu:GetSelection() ~= nil ) then
			self.quickslotsMenu:DisplaySettings( true );
		else
			self.quickslotsMenu:RefreshComboBox();
		end
		if ( settings.barMode ~= NORMAL_MODE ) then
			settings.barMode = QUICKSLOT_MODE;
		else
			self.priorBarMode = QUICKSLOT_MODE;
		end
	elseif ( mode == EXTENSION_MODE ) then
		self:ClearItems();
		self:AddItem( self.selectionBox );
		self:AddItem( self.extensionsMenu );
		
		if ( self.extensionsMenu.barList:GetSelection() ~= nil ) then
			self.extensionsMenu:DisplaySettings( true );
		else
			self.extensionsMenu:RefreshComboBox();
		end
		if ( settings.barMode ~= NORMAL_MODE ) then
			settings.barMode = EXTENSION_MODE;
		else
			self.priorBarMode = EXTENSION_MODE;
		end
	elseif ( mode == INVENTORY_MODE ) then
		self:ClearItems();
		self:AddItem( self.selectionBox );
		self:AddItem( self.inventoryMenu );
		
		if ( self.inventoryMenu:GetSelection() ~= nil ) then
			self.inventoryMenu:DisplaySettings( true );
		else
			self.inventoryMenu:RefreshComboBox();
		end
		if ( settings.barMode ~= NORMAL_MODE ) then
			settings.barMode = QUICKSLOT_MODE;
		else
			self.priorBarMode = QUICKSLOT_MODE;
		end
	elseif ( mode == GENERAL_MODE ) then
		self:ClearItems();
		self:AddItem( self.selectionBox );
		self:AddItem( self.generalMenu );
		self.generalMenu:DisplaySettings( true );
		self.generalMenu:RefreshComboBox();
		if ( settings.barMode ~= NORMAL_MODE ) then
			settings.barMode = QUICKSLOT_MODE;
		else
			self.priorBarMode = QUICKSLOT_MODE;
		end
	elseif ( mode == ABOUT_MODE ) then
		self:ClearItems();
		self:AddItem( self.selectionBox );
		self:AddItem( self.aboutMenu );
		self.aboutMenu:DisplaySettings( true );
		if ( settings.barMode ~= NORMAL_MODE ) then
			settings.barMode = QUICKSLOT_MODE;
		else
			self.priorBarMode = QUICKSLOT_MODE;
		end
	end
	self.mode = mode;
	maitre_d:RefreshBars();
end

function MainMenu:RegenerateQuickslotsMenu()
	self.quickslotsMenu = Tonic.TonicBars.Menu.QuickslotsMenu();
	self:SetupMainMenuDisplay( QUICKSLOT_MODE );
end

function MainMenu:RegenerateEasyBarsMenu()
	self.easybarMenu = Tonic.TonicBars.Menu.EasyBarMenu();
	self:SetupMainMenuDisplay( EASYBAR_MODE );
end

function MainMenu:GetSelection()
	if ( self.mode == EASYBAR_MODE and self.easybarMenu ~= nil and self.easybarMenu.barList ~= nil) then
		return self.easybarMenu.barList:GetSelection();
	elseif ( self.mode == QUICKSLOT_MODE and self.easybarMenu ~= nil and self.quickslotsMenu.barList ~= nil ) then
		return self.quickslotsMenu.barList:GetSelection();
	elseif ( self.mode == EXTENSION_MODE and self.easybarMenu ~= nil and self.extensionsMenu.barList ~= nil ) then
		return self.extensionsMenu.barList:GetSelection();
	elseif ( self.mode == INVENTORY_MODE and self.easybarMenu ~= nil and self.inventoryMenu.barList ~= nil ) then
		return self.inventoryMenu.barList:GetSelection();
	else
		return nil;
	end
end

function MainMenu:RefreshComboBox()
	if ( self.mode == EASYBAR_MODE ) then
		return self.easybarMenu:RefreshComboBox( self:GetSelection() );
	elseif ( self.mode == QUICKSLOT_MODE ) then
		return self.quickslotsMenu:RefreshComboBox( self:GetSelection() );
	elseif ( self.mode == EXTENSION_MODE ) then
		return self.extensionsMenu:RefreshComboBox( self:GetSelection() );
	elseif ( self.mode == INVENTORY_MODE ) then
		return self.inventoryMenu:RefreshComboBox( self:GetSelection() );
	elseif ( self.mode == GENERAL_MODE ) then
		return nil;
	elseif ( self.mode == ABOUT_MODE ) then
		return nil;
	end
end

function MainMenu:EnableTriggers( enabled )
	if ( self.mode == EASYBAR_MODE ) then
		return self.easybarMenu:EnableTriggers( enabled );
	elseif ( self.mode == QUICKSLOT_MODE ) then
		return self.quickslotsMenu:EnableTriggers( enabled );
	elseif ( self.mode == EXTENSION_MODE ) then
		return self.extensionsMenu:EnableTriggers( enabled );
	elseif ( self.mode == INVENTORY_MODE ) then
		return self.inventoryMenu:EnableTriggers( enabled );
	elseif ( self.mode == GENERAL_MODE ) then
		return nil;
	elseif ( self.mode == ABOUT_MODE ) then
		return nil;
	end
end

function MainMenu:GetMode()
	return self.mode;
end

function MainMenu:BarClicked( selectedBarId )
	local settings = theSettingsManager:GetSettings();
	local barSettings = theSettingsManager:GetBarSettings( selectedBarId );
	if ( self.mode ~= NORMAL_MODE and barSettings.barType == QUICKSLOTBAR ) then
		self:SetupMainMenuDisplay( QUICKSLOT_MODE );
		self.quickslotsMenu:RefreshComboBox( selectedBarId );
	elseif ( self.mode ~= NORMAL_MODE and barSettings.barType == EXTENSIONBAR ) then
		self:SetupMainMenuDisplay( EXTENSION_MODE );
		self.extensionsMenu:RefreshComboBox( selectedBarId );
	elseif ( self.mode ~= NORMAL_MODE and (barSettings.barType == TABBED_INV_BAR or barSettings.barType == WINDOW_INV_BAR) ) then
		self:SetupMainMenuDisplay( INVENTORY_MODE );
		self.inventoryMenu:RefreshComboBox( selectedBarId );
	end
end